package app.views.table;

import app.components.label.ValidationLabel;
import app.components.navigation.NavigationHeader;
import app.data.bean.PlayerForTable;
import app.data.service.MatchService;
import app.data.service.MatchdayService;
import app.data.service.PlayerService;
import app.data.service.SeasonService;
import app.utils.StringUtils;
import app.utils.VaadinUtils;
import app.navigation.Navigation;
import app.views.main.MainView;
import app.views.navigation.NavigationViewBase;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.router.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.NoSuchElementException;

@CssImport("app/views/table/table-view.css")
@Route(value = "table", layout = MainView.class)
@RouteAlias(value = "", layout = MainView.class)
@PageTitle("Schachliga DACH - Table")
public class TableView extends NavigationViewBase {

    private final PlayerService playerService;

    private final Grid<PlayerForTable> grid = new Grid<>();

    public TableView(@Autowired SeasonService seasonService,
                     @Autowired MatchdayService matchdayService,
                     @Autowired MatchService matchService,
                     @Autowired PlayerService playerService) {
        super(seasonService, matchdayService, matchService, true, true, false);
        this.playerService = playerService;

//        this.navigation = new Navigation("table", seasonService, matchdayService, matchService, true);
        this.navigation.setOnlyMatchdaysWithActivity(true);
        this.navigation.setSeasonEnabled(true);
        this.navigation.setMatchdayEnabled(true);

        addClassName("table-view");
        addClassName("content");
        configureLayout();
    }

    ////////////
    // LAYOUT //
    ////////////

    private void configureLayout() {
        Div gridWrapper = new Div(grid);
        gridWrapper.addClassName("grid-wrapper");

        add(new NavigationHeader(navigation), new HorizontalLayout(gridWrapper));

        configureGrid();
    }

    protected void configureGrid() {
        // TODO: add background color for content
        // TODO: add diff to last matchday

        Label headerPlace = new Label("Place");
        headerPlace.addClassName("important-table-column-header");

        Label headerPlayer = new Label("Player");
        headerPlayer.addClassName("important-table-column-header");

        Label headerMatchesPlayed = new Label("Played");

        Label headerMatchPoints = new Label("Points");
        headerMatchPoints.addClassName("important-table-column-header");

        Label headerWon = new Label("W");

        Label headerDrawn = new Label("D");

        Label headerLost = new Label("L");

        Label headerGames = new Label("Games");

        Label headerDiff = new Label("Diff");

        grid.addColumn(VaadinUtils.getBoldStringRenderer(PlayerForTable::getPlaceString))
                .setHeader(headerPlace)
                .setTextAlign(ColumnTextAlign.CENTER)
//                .setClassNameGenerator(playerForTable -> "leftmost-column-cell")
                .setWidth("5em");

        grid.addColumn(VaadinUtils.getPlayerRenderer(PlayerForTable::getPlayer))
                .setHeader(headerPlayer)
                .setTextAlign(ColumnTextAlign.START)
                .setWidth("13em");

        grid.addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getAmountOfMatches)
                .setHeader(headerMatchesPlayed)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("5em");

        grid.addColumn(VaadinUtils.getBoldStringRenderer(player -> String.valueOf(player.getMatchPoints())))
                .setHeader(headerMatchPoints)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getAmountOfMatchesWon)
                .setHeader(headerWon)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getAmountOfMatchesDrawn)
                .setHeader(headerDrawn)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getAmountOfMatchesLost)
                .setHeader(headerLost)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        grid.addColumn((ValueProvider<PlayerForTable, String>) this::getResultString)
                .setHeader(headerGames)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addColumn((ValueProvider<PlayerForTable, String>) player -> StringUtils.getSignedString(player.getGamePointDiff()))
                .setHeader(headerDiff)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("5em");

        grid.setWidth("51em");
        grid.setHeightByRows(true);

        grid.addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);
    }

    private String getResultString(PlayerForTable player) {
        return StringUtils.getResultString(player.getGamePointsForSelf(), player.getGamePointsForOpponents());
    }

    /////////////
    // CONTENT //
    /////////////

    @Override
    public String getRoute() {
        return "table";
    }

    @Override
    protected void configureContent() {
        try {
            grid.setItems(playerService.getPlayersForTable(navigation.getSelectedMatchday().orElseThrow()));
        } catch (NoSuchElementException e) {
            removeAll();
            add(navigation.getValidationLabel());
        }
    }
}
